@@ -61,13 +61,13 @@ def _command_is_valid(cmd):
         cmd_abspath = os.path.abspath(os.path.expanduser(cmd))
     except (FileNotFoundError, OSError):
         return False
-    return cmd in builtins.__xonsh_commands_cache__ or (
+    return cmd in builtins.__xonsh__.commands_cache or (
         os.path.isfile(cmd_abspath) and os.access(cmd_abspath, os.X_OK)
     )
 
 
 def _command_is_autocd(cmd):
-    if not builtins.__xonsh_env__.get("AUTO_CD", False):
+    if not builtins.__xonsh__.env.get("AUTO_CD", False):
         return False
     try:
         cmd_abspath = os.path.abspath(os.path.expanduser(cmd))
@@ -105,16 +105,20 @@ class XonshLexer(PythonLexer):
     filenames = ["*.xsh", "*xonshrc"]
 
     def __init__(self, *args, **kwargs):
-        # If the lexor is loaded as a pygment plugin, we have to mock
-        # __xonsh_env__ and __xonsh_commands_cache__
-        if not hasattr(builtins, "__xonsh_env__"):
-            setattr(builtins, "__xonsh_env__", {})
+        # If the lexer is loaded as a pygment plugin, we have to mock
+        # __xonsh__.env and __xonsh__.commands_cache
+        if not hasattr(builtins, "__xonsh__"):
+            from argparse import Namespace
+
+            setattr(builtins, "__xonsh__", Namespace())
+        if not hasattr(builtins.__xonsh__, "env"):
+            setattr(builtins.__xonsh__, "env", {})
             if ON_WINDOWS:
                 pathext = os_environ.get("PATHEXT", [".EXE", ".BAT", ".CMD"])
-                builtins.__xonsh_env__["PATHEXT"] = pathext.split(os.pathsep)
-        if not hasattr(builtins, "__xonsh_commands_cache__"):
-            setattr(builtins, "__xonsh_commands_cache__", CommandsCache())
-        _ = builtins.__xonsh_commands_cache__.all_commands  # NOQA
+                builtins.__xonsh__.env["PATHEXT"] = pathext.split(os.pathsep)
+        if not hasattr(builtins.__xonsh__, "commands_cache"):
+            setattr(builtins.__xonsh__, "commands_cache", CommandsCache())
+        _ = builtins.__xonsh__.commands_cache.all_commands  # NOQA
         super().__init__(*args, **kwargs)
 
     tokens = {
@@ -325,8 +329,8 @@ def partial_color_tokenize(template):
     of tuples mapping the token to the string which has that color.
     These sub-strings maybe templates themselves.
     """
-    if hasattr(builtins, "__xonsh_shell__"):
-        styles = __xonsh_shell__.shell.styler.styles
+    if hasattr(builtins.__xonsh__, "shell"):
+        styles = __xonsh__.shell.shell.styler.styles
     else:
         styles = None
     color = Color.NO_COLOR
@@ -452,7 +456,7 @@ class XonshStyle(Style):
                     file=sys.stderr,
                 )
                 value = "default"
-                builtins.__xonsh_env__["XONSH_COLOR_STYLE"] = value
+                builtins.__xonsh__.env["XONSH_COLOR_STYLE"] = value
         cmap = STYLES[value]
         if value == "default":
             self._smap = XONSH_BASE_STYLE.copy()
@@ -466,10 +470,10 @@ class XonshStyle(Style):
         self._style_name = value
         # Convert new ansicolor names to old PTK1 names
         # Can be remvoed when PTK1 support is dropped.
-        if builtins.__xonsh_shell__.shell_type != "prompt_toolkit2":
+        if builtins.__xonsh__.shell.shell_type != "prompt_toolkit2":
             for smap in [self.trap, cmap, PTK_STYLE, self._smap]:
                 smap.update(ansicolors_to_ptk1_names(smap))
-        if ON_WINDOWS and "prompt_toolkit" in builtins.__xonsh_shell__.shell_type:
+        if ON_WINDOWS and "prompt_toolkit" in builtins.__xonsh__.shell.shell_type:
             self.enhance_colors_for_cmd_exe()
 
     @style_name.deleter
@@ -481,7 +485,7 @@ class XonshStyle(Style):
             When using the default style all blue and dark red colors
             are changed to CYAN and intense red.
         """
-        env = builtins.__xonsh_env__
+        env = builtins.__xonsh__.env
         # Ensure we are not using ConEmu or Visual Stuio Code
         if "CONEMUANSI" in env or "VSCODE_PID" in env:
             return
